--[[--------------------------------------------------------------------------------
  Class Setup
-----------------------------------------------------------------------------------]]
-- Note, le nom de l'addon est TRES important, il doit etre alphabetiquement devant EP_SelfCast


EP_AnnounceCast = AceLibrary("AceAddon-2.0"):new(
	"AceHook-2.1",
	"AceConsole-2.0",
	"AceDB-2.0",
	"AceEvent-2.0"
)

local L = AceLibrary("AceLocale-2.2"):new("EP_AnnounceCast")
local G = AceLibrary("Gratuity-2.0")
local BS = AceLibrary("Babble-Spell-2.2")

EP_AnnounceCast.defaults = {
	msg = "%t ====> %s #(%d secs) #(%r@ heal %h@)",
	me = true,
	outside = true,
	outside_m = "whisper",
	party = true,
	party_m = "party",
	raid = true,
	raid_m = "raid",
	instant = true,
	delay = "2.5",
	spells = {
		["PRIEST"] = {
			['Lesser Heal'] = 1,
			['Heal'] = 1,
			['Greater Heal'] = 1,
			['Flash Heal'] = 2,
			['Renew'] = 1,
			['Resurrection'] = 4,
			['Power Word: Fortitude'] = 0,
			['Power Word: Shield'] = 1,
			['Dispel Magic'] = 0,
			['Cure Disease'] = 0,
			['Abolish Disease'] = 0,
			['Divine Spirit'] = 1,
			['Prayer of Healing'] = 1,
			['Prayer of Fortitude'] = 1,
			['Prayer of Spirit'] = 1,
			['Prayer of Shadow Protection'] = 1,
			['Shadow Protection'] = 1,
			['Holy Nova'] = 1,
			['Power Infusion'] = 1,
			['Lightwell'] = 1,
			['Shackle Undead'] = 4,
			['Holy Nova'] = 1
		} ,
		["PALADIN"] = {
		      ['Holy Light'] = 1,
		      ['Flash of Light'] = 1,
		      ['Purify'] = 1,
		      ['Cleanse'] = 1,
		      ['Redemption'] = 1,
		      ['Lay on Hands'] = 1,
		      ['Devotion Aura'] = 1,
		      ['Divine Protection'] = 1,
		      ['Blessing of Might'] = 1,
		      ['Blessing of Wisdom'] = 1,
		      ['Blessing of Light'] = 1,
		      ['Blessing of Sacrifice'] = 1,
		      ['Blessing of Protection'] = 1,
		      ['Blessing of Salvation'] = 1,
		      ['Blessing of Sanctuary'] = 1,
		      ['Blessing of Freedom'] = 1,
		},
		["DRUID"] = {
		      ['Thorns'] = 1,
		      ['Mark of the Wild'] = 1,
		      ['Gift of the Wild'] = 1,
		      ['Rejuvenation'] = 1,
		      ['Regrowth'] = 1,
		      ['Healing Touch'] = 1,
		      ['Tranquility'] = 1,
		      ['Remove Curse'] = 1,
		      ['Cure Poison'] = 1,
		      ['Abolish Poison'] = 1,
		      ['Rebirth'] = 1,
		      ['Innervate'] = 1
		}
	}
}

function EP_AnnounceCast:OnInitialize()
	_, self.MyClass = UnitClass("player")

	self:RegisterDB("EP_AnnounceCastDB")
	self:RegisterDefaults('class', self.defaults)
	self:InitMenu()
	self.GSpells = {
		-- priest
		[BS["Holy Nova"]] = 1,
		[BS["Prayer of Healing"]] = 1,
		[BS["Prayer of Fortitude"]] = 1,
		[BS["Prayer of Spirit"]] = 1,
		[BS["Prayer of Shadow Protection"]] = 1,
		
		-- druid
		[BS["Gift of the Wild"]] = 1,
		[BS["Tranquility"]] = 1
	}

	self.SpellData = {}
end


--[[--------------------------------------------------------------------------------
  Addon Enabling/Disabling
-----------------------------------------------------------------------------------]]

function EP_AnnounceCast:OnEnable()
	self:RegisterEvent("UNIT_SPELLCAST_START", "EVENT_SPELLCAST")
	self:RegisterEvent("UNIT_SPELLCAST_STOP", "EVENT_SPELLCAST")
	self:RegisterEvent("UNIT_SPELLCAST_FAILED", "EVENT_SPELLCAST_CANCEL")
	self:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED", "EVENT_SPELLCAST_CANCEL")
	
	self:RegisterEvent("UNIT_SPELLCAST_SENT", "EVENT_SPELLCAST_SENT")
end


--[[--------------------------------------------------------------------------------
  Internal Functions
-----------------------------------------------------------------------------------]]

local function nameToUnit(name)
	local i = 1
	local nbr = GetNumPartyMembers()

	if UnitName("player") == name then
		return "player"
	end

	while (i <= nbr) do
		if UnitName("party"..i) == name then
			return "party"..i
		end
		
		if UnitName("partypet"..i) == name then
			return "partypet"..i
		end

		i = i+1
	end

	local i = 1
	local nbr = GetNumRaidMembers()

	while (i <= nbr) do
		if UnitName("raid"..i) == name then
			return "raid"..i
		end
		
		if UnitName("raidpet"..i) == name then
			return "raidpet"..i
		end

		i = i+1
	end

	if UnitName("target") == name then
		return "target"
	end

	if UnitName("mouseover") == name then
		return "mouseover"
	end

	return nil
end


--[[--------------------------------------------------------------------------------
   Main Processing
-----------------------------------------------------------------------------------]]

function EP_AnnounceCast:Announce()
	local msg, method
	local display = 0
		
	if BS:HasReverseTranslation(self.SpellData.name) then
		display = self.db.class.spells[self.MyClass][BS:GetReverseTranslation(self.SpellData.name)]
	end
		
	-- Le sort est pas definis, ou n'est jamais affich
	if display == 0 or display == nil then
		return
	end

	if BS:HasReverseTranslation(self.SpellData.name) and
		BS:GetReverseTranslation(self.SpellData.name) == "Resurrection" and
		not UnitIsDeadOrGhost(self.SpellData.target or "") and
		GameTooltip:IsVisible() then

		self.SpellData.targetName = GameTooltipTextLeft1:GetText()
		
		-- Cadavre d'Hurgan 
		-- string.find("Cadavre d'Hurgan", "([^ ']+)$")
		-- Cadavre de oxman 
		-- string.find("Cadavre de oxman", "([^ ']+)$")
		-- Garde de Pierre Centurus - Arathi
		-- string.find("Garde de Pierre Centurus - Arathi", "([^ ']+) %- .+")

		local _, _, a = string.find(self.SpellData.targetName, "([^ ']+)$")
		DEFAULT_CHAT_FRAME:AddMessage(tostring(self.SpellData.targetName).." donne "..tostring(a))
		if a then
			self.SpellData.targetName = a
		end
		
		local _, _, a = string.find(self.SpellData.targetName, "([^ ']+) %- .+$")
		if a then
			self.SpellData.targetName = a
		end

		local _, _, a = string.find(self.SpellData.targetName, ".+|(.+)|r")
		if a then
			self.SpellData.targetName = a
		end

		self.SpellData.target = nameToUnit(self.SpellData.targetName)
	end

	--DEFAULT_CHAT_FRAME:AddMessage("target : "..tostring(self.SpellData.target))
	if self.SpellData.target then
		_, self.SpellData.targetServer = UnitName(self.SpellData.target)
	end
	
	--DEFAULT_CHAT_FRAME:AddMessage("targetserver : "..tostring(self.SpellData.targetServer))
	

	-- On lance le sort sur nous mais on veut pas l'afficher
	if UnitName("player") == self.SpellData.targetName and not self.db.class.me then
		return
	end

	if not UnitIsPlayer(self.SpellData.target or "player") then
		return
	end

	-- Test aussi sur le nom de la cible car UnitIsPlayer ne semble pas marcher lors d'un ciblage passif sur sois meme
	if self.SpellData.name and not UnitIsPlayer(self.SpellData.target) and not UnitName('player') == self.SpellData.targetName and not self.GSpells[self.SpellData.name] then
		return
	end


	if self.SpellData.targetName == UNKNOWNOBJECT then
		return
	end

	if self.SpellData.delay and display ~= 4 then
		local delay = string.gsub(self.SpellData.delay, ",", "%.")
		if (tonumber(delay) or 0) < tonumber(self.db.class.delay) then
			return
		end
	end

	
	if not self.SpellData.delay and not self.db.class.instant and display ~= 4 then
		return
	end


	if self.SpellData.name and self.SpellData.targetName then
		msg = self.db.class.msg

		if self.GSpells[self.SpellData.name] then
			msg = string.gsub(msg, " ?%%t ?", "")
			-- it's for /whisper
			self.SpellData.targetName = UnitName("player")
		else
			msg = string.gsub(msg, "%%t", self.SpellData.targetName or "")
		end

		msg = string.gsub(msg, "%%s", self.SpellData.name)
		if self.SpellData.delay then
			msg = string.gsub(msg, "%%d", self.SpellData.delay or "")
			msg = string.gsub(msg, "#", "")
		else
			msg = string.gsub(msg, "#.+#", "")
		end

		msg = string.gsub(msg, "%%r", self.SpellData.rank or "")

		if self.SpellData.heal and self.SpellData.heal ~= 0 then
			msg = string.gsub(msg, "%%h", self.SpellData.heal)
			msg = string.gsub(msg, "@", "")
		else
			msg = string.gsub(msg, "@.+@", "")
		end

		-- In Raid :-)
		if UnitInRaid(self.SpellData.target) and display ~= 2 then
			if not self.db.class.raid then
				return
			end

			self:SendMessage(msg, self.db.class.raid_m)
			return
		end

		-- In Party :-)
		if UnitInParty(self.SpellData.target) and GetNumPartyMembers() > 0 and display ~= 3 then
			if not self.db.class.party then
				return
			end

			self:SendMessage(msg, self.db.class.party_m)
			return
		end

		-- Outside Raid/Party :-)
		if not self.db.class.outside then
			return
		end
		
		if display ~= 1 and display ~= 4 then
			return
		end

		method = self.db.class.outside_m
		self:SendMessage(msg , method)
	end

	self.SpellData = {}
end

function EP_AnnounceCast:SendMessage(msg, method)
	if method == "whisper" then
		msg = string.gsub(msg, " ?"..self.SpellData.targetName.." ?", "")
		
		local target = self.SpellData.targetName
		if (self.SpellData.targetServer) then
			target = target.."-"..self.SpellData.targetServer
		end
--DEFAULT_CHAT_FRAME:AddMessage("target : "..tostring(target))
		
		SendChatMessage(msg , method, nil, target)
	elseif method ~= "party" and method ~= "raid" and method ~= "say" then
		if GetChannelName(method) then
			SendChatMessage(msg, "CHANNEL", nil, GetChannelName(method))
		end
	else
		SendChatMessage(msg, method)
	end
end

function EP_AnnounceCast:ParseTooltip(type, id)
	local heal, heal_min, heal_max, uber

	G:Erase()

	if type == "action" then
		G:SetAction(id)
		-- no Rank in tooltips from BOOKTYPE_SPELL
		self.SpellData.rank = G:GetLine(1, true)
		_,_, self.SpellData.rank = string.find(self.SpellData.rank or "", ".+ (.+)")
	elseif type == "spellbook" then
		G:SetSpell(id, BOOKTYPE_SPELL)
	end

	self.SpellData.name = G:GetLine(1)
	self.SpellData.delay = G:GetLine(3)
	self.SpellData.desc = G:GetLine(4)

	-- Le sort est pas definis, on n'affiche rien
	if not BS:HasReverseTranslation(self.SpellData.name) then
		self.SpellData = {}
		return
	end

	_, _, self.SpellData.delay = string.find(self.SpellData.delay or "", "^([%d.,]+)")

	if self.SpellData.desc then
		-- Soigne la cible de w  x points de vie et ensuite de y points de vie en z secondes
		_, _, heal_min, heal_max = string.find(self.SpellData.desc, L["pattern"][1])


		if not heal_min then
			-- Soigne de x points de vie la cible en y sec
			_, _, heal = string.find(self.SpellData.desc, L["pattern"][2])
			-- le sort soigne sur le temps, donc il soigne instantannment 0 de vie
			if heal then
				heal = 0
			end
		end

		if not heal and not heal_min then
			-- "w  x points de vie"
			_, _, heal_min, heal_max = string.find(self.SpellData.desc, L["pattern"][3])
		end


		if not heal and not heal_min then
			-- "x points de vie"
			_, _, heal = string.find(self.SpellData.desc, L["pattern"][4])
		end

		if heal_min then
			heal = (tonumber(heal_min) + tonumber(heal_max))/2
		end

		self.SpellData.heal = heal
	end
end

function EP_AnnounceCast:GetSpellId(name, rank)
	local nameBook, rankBook
	local i = 1
	local find = nil

	repeat
		nameBook, rankBook = GetSpellName(i, BOOKTYPE_SPELL)
		_,_, rankBook = string.find(rankBook or "", ".+ (.+)")
		rankBook = tonumber(rankBook)

		if name == nameBook then
			if rank == rankBook or rank == 999 or not rankBook then
				find = true
				-- set the rank here, because no rank in tooltips
				self.SpellData.rank = rankBook
			end
		end

		if name ~= nameBook and find then
			return i-1
		end

		i = i+1
	until not nameBook

	return nil
end


--[[--------------------------------------------------------------------------------
  Event handlers
-----------------------------------------------------------------------------------]]

function EP_AnnounceCast:EVENT_SPELLCAST_SENT()
	self.SpellData.name = arg2
	self.SpellData.rank = arg3
	self.SpellData.target = nameToUnit(arg4)
	self.SpellData.targetName = arg4
end

function EP_AnnounceCast:EVENT_SPELLCAST()
	self:Announce()

	self.SpellData = {}
end

function EP_AnnounceCast:EVENT_SPELLCAST_CANCEL()
	self.SpellData = {}
end


--[[---------------------------------------------------------------------------------
  Options
------------------------------------------------------------------------------------]]

function EP_AnnounceCast:OptionDelay(value)
	if type(value) == "nil" then return self.db.class.delay end
	self.db.class.delay = value
	self.ddoptions.args.delay.desc = "Announce instant spell with a delay higher than "..self.db.class.delay
end

function EP_AnnounceCast:ToggleInstant(value)
	if type(value) == "nil" then return self.db.class.instant end
	self.db.class.instant = value
end

function EP_AnnounceCast:ToggleMe(value)
	if type(value) == "nil" then return self.db.class.me end
	self.db.class.me = value
end

function EP_AnnounceCast:OptionMsg(value)
	if type(value) == "nil" then return self.db.class.msg end
	self.db.class.msg = value
end

function EP_AnnounceCast:ToggleParty(value)
	if type(value) == "nil" then return self.db.class.party end
	self.db.class.party = value
end

function EP_AnnounceCast:OptionPartyMethod(value)
	if type(value) == "nil" then return self.db.class.party_m end
	self.db.class.party_m = value
end

function EP_AnnounceCast:ToggleRaid(value)
	if type(value) == "nil" then return self.db.class.raid end
	self.db.class.raid = value
end

function EP_AnnounceCast:OptionRaidMethod(value)
	if type(value) == "nil" then return self.db.class.raid_m end
	self.db.class.raid_m = value
end

function EP_AnnounceCast:ToggleOutside(value)
	if type(value) == "nil" then return self.db.class.outside end
	self.db.class.outside = value
end

function EP_AnnounceCast:OptionOutsideMethod(value)
	if type(value) == "nil" then return self.db.class.outside_m end
	self.db.class.outside_m = value
end

function EP_AnnounceCast:OptionSpell(spell, value, set)
	if set == 0 then return self.db.class.spells[self.MyClass][spell] == value  end
	self.db.class.spells[self.MyClass][spell] = value
	self:InitMenu()
end


--[[---------------------------------------------------------------------------------
  Menu
------------------------------------------------------------------------------------]]

function EP_AnnounceCast:InitMenu()
	self.dewdrop = AceLibrary("Dewdrop-2.0")

	local spells_never = {}
	local spells_normal = {}
	local spells_always = {}
	local spells_party = {}
	local spells_raid = {}
	local empty = {
		empty = {
			name = "empty",
			type = "execute",
			desc = "empty list",
			func = function() return nil end,
			disabled = true
		}
	}

	local tmp = self.db.class.spells[self.MyClass]
	if not tmp then
		tmp = {}
	end

	foreach(tmp,
		function(k,v)
			local value = {
				name = BS[k],
				type = "group",
				desc = "Announce for this spell",
				args = {
					never = {
						name = "never",
						type = "toggle",
						desc = "Never mode",
						get = function() return self:OptionSpell(k, 0, 0) end,
						set = function() return self:OptionSpell(k, 0, 1) end
					},
					normal = {
						name = "normal",
						type = "toggle",
						desc = "Normal mode",
						get = function() return self:OptionSpell(k, 1, 0) end,
						set = function() return self:OptionSpell(k, 1, 1) end
					},
					party = {
						name = "party",
						type = "toggle",
						desc = "Party mode",
						get = function() return self:OptionSpell(k, 2, 0) end,
						set = function() return self:OptionSpell(k, 2, 1) end
					},
					raid = {
						name = "raid",
						type = "toggle",
						desc = "Raid mode",
						get = function() return self:OptionSpell(k, 3, 0) end,
						set = function() return self:OptionSpell(k, 3, 1) end
					},
					always = {
						name = "always",
						type = "toggle",
						desc = "Always mode",
						get = function() return self:OptionSpell(k, 4, 0) end,
						set = function() return self:OptionSpell(k, 4, 1) end
					}
				}
			}

			key = string.gsub(k, " ", "")

			if v == 0 then
				spells_never[key] = value
			elseif v == 1 then
				spells_normal[key] = value
			elseif v == 2 then
				spells_party[key] = value
			elseif v == 3 then
				spells_raid[key] = value
			elseif v == 4 then
				spells_always[key] = value
			end
		end
	)

	if next(spells_never) == nil then
		spells_never = nil
	end

	if next(spells_normal) == nil then
		spells_normal = nil
	end

	if next(spells_party) == nil then
		spells_party = nil
	end

	if next(spells_raid) == nil then
		spells_raid = nil
	end

	if next(spells_always) == nil then
		spells_always = nil
	end


	self.ddoptions = {
		type = "group",
		args = {
			delay = {
				name	= "delay",
				type	= "range",
				desc	= "Announce instant spell with a delay higher than "..self.db.class.delay,
				get	= function() return self:OptionDelay() end,
				set	= function(v) return self:OptionDelay(v) end,
				min = 0,
				max = 5,
				step = 0.5
			},
			instant = {
				name	= "instant",
				type	= "toggle",
				desc	= "Announce instant spell",
				get	= function() return self:ToggleInstant() end,
				set	= function(v) return self:ToggleInstant(v) end,
			},
			me = {
				name	= "me",
				type	= "toggle",
				desc	= "Announce spell you cast on yourself",
				get	= function() return self:ToggleMe() end,
				set	= function(v) return self:ToggleMe(v) end,
			},
			msg = {
				name	= "msg",
				type	= "text",
				desc	= "Announce format",
				get	= function() return self:OptionMsg() end,
				set	= function(v) return self:OptionMsg(v) end,
				usage = "%s spellname, %r rank, %t target, %h heal average, %d casting time",
			},
			outside = {
				name = "outside",
				type = "group",
				desc = "Outside option",
				args = {
					enable = {
						name = "enable",
						type = "toggle",
						desc = "Enable raid announce",
						get	= function() return self:ToggleRaid() end,
						set	= function(v) return self:ToggleRaid(v) end,
					},
					method = {
						name = "method",
						type = "text",
						desc = "Method for the announce (say, whisper)",
						get	= function() return self:OptionOutsideMethod() end,
						set	= function(v) return self:OptionOutsideMethod(v) end,
						usage = "< say | whisper >",
					},
				},
			},
			party = {
				name = "party",
				type = "group",
				desc = "Party option",
				args = {
					enable = {
						name = "enable",
						type = "toggle",
						desc = "Enable party announce",
						get	= function() return self:ToggleParty() end,
						set	= function(v) return self:ToggleParty(v) end,
					},
					method = {
						name = "method",
						type = "text",
						desc = "Method for the announce (say, whisper, party, raid, channelName)",
						get	= function() return self:OptionPartyMethod() end,
						set	= function(v) return self:OptionPartyMethod(v) end,
						usage = "< say | whisper | party | raid | channelName >",
					},
				},
			},
			raid = {
				name = "raid",
				type = "group",
				desc = "Raid option",
				args = {
					enable = {
						name = "enable",
						type = "toggle",
						desc = "Enable raid announce",
						get	= function() return self:ToggleRaid() end,
						set	= function(v) return self:ToggleRaid(v) end,
					},
					method = {
						name = "method",
						type = "text",
						desc = "Method for the announce (say, whisper, party, raid, channelName)",
						get	= function() return self:OptionRaidMethod() end,
						set	= function(v) return self:OptionRaidMethod(v) end,
						usage = "< say | whisper | party | raid | channelName >",
					},
				},
			},
			spells = {
				name = "spells",
				type	="group",
				desc = "Spells list",
				args = {
					never = {
						name = "never",
						type = "group",
						desc = "Spells never shown",
						args = spells_never or empty
					},
					normal = {
						name = "normal",
						type = "group",
						desc = "Spells shown according to delay and instant option",
						args = spells_normal or empty
					},
					party = {
						name = "party",
						type = "group",
						desc = "Spells shown when you're in a party",
						args = spells_party or empty
					},
					raid = {
						name = "raid",
						type = "group",
						desc = "Spells shown when you're in a raid",
						args = spells_raid or empty
					},
					always = {
						name = "always",
						type = "group",
						desc = "Spells always shown",
						args = spells_always or empty
					}
				}
			}
		}
	}

	self.slashOptions = {
		type = "group",
		args = {
			config = {
				name = "config",
				desc = "Open config menu",
				type = "execute",
				func = function()
					self.dewdrop:InjectAceOptionsTable(self, self.ddoptions)
					self.dewdrop:Register(UIParent, 'children', self.ddoptions, 'cursorX', true, 'cursorY', true)
					self.dewdrop:Open(UIParent)
				end,
			},
		}
	}

	self:RegisterChatCommand({"/epac", "/epannouncecast"}, self.slashOptions)
	self:TriggerEvent("EP_ANNOUNCECAST_UPDATE")
end

