--[[--------------------------------------------------------------------------------
  Class Setup
-----------------------------------------------------------------------------------]]

EP_VoileMortel = AceLibrary("AceAddon-2.0"):new(
	"AceDB-2.0",
	"AceEvent-2.0",
	"AceConsole-2.0"
)

EP_VoileMortel.defaults = {
	show = "combat",
	display = "side",
	health1 = 50,
	health2 = 25,
}


function EP_VoileMortel:OnInitialize()
	self:RegisterDB("EP_VoileMortelDB")
	self:RegisterDefaults('profile', EP_VoileMortel.defaults)
	self:InitMenu()

	self.so_fit = 0.2
	self.so_fot = 0.8
	self.so_fiht = 0.5
	self.so_foht = 0

	self.st_fit = 0.1
	self.st_fot = 0.4
	self.st_fiht = 0.25
	self.st_foht = 0

	EP_VoileMortelFrame:SetAlpha(0)
	self:SetMask()
	self.Flash = nil
end


--[[--------------------------------------------------------------------------------
  Addon Enabling/Disabling
-----------------------------------------------------------------------------------]]

function EP_VoileMortel:OnEnable()
	self:RegisterEvent("UNIT_HEALTH")
end


--[[--------------------------------------------------------------------------------
  Internal Functions
-----------------------------------------------------------------------------------]]

function EP_VoileMortel:FormatToRange(arg, lower_bound, upper_bound, variable)
	arg = tonumber(arg)
	if not arg then
		arg = DEFAULT_OPTIONS[variable]
	elseif arg > upper_bound then
		arg = upper_bound
	elseif arg < lower_bound then
		arg = lower_bound
	end

	return arg
end

function EP_VoileMortel:SetMask()
	if self.db.profile.layout == "full" then
		EP_VoileMortelFrameTexture:SetTexture("Interface\\AddOns\\EP_VoileMortel\\Texture\\Full-Mask")
	elseif self.db.profile.layout == "side" then
		EP_VoileMortelFrameTexture:SetTexture("Interface\\AddOns\\EP_VoileMortel\\Texture\\Sides-Mask")
	else
		EP_VoileMortelFrameTexture:SetTexture("Interface\\FullScreenTextures\\LowHealth")
	end
end


--[[--------------------------------------------------------------------------------
  Events
-----------------------------------------------------------------------------------]]

function EP_VoileMortel:UNIT_HEALTH()
	if arg1 == "player" then
		local health = UnitHealth("player")*100/UnitHealthMax("player")
		if UnitIsDeadOrGhost("player") or health > self.db.profile.health1 or (self.db.profile.show == "combat" and not UnitAffectingCombat("player")) then
			UIFrameFlashRemoveFrame(EP_VoileMortelFrame)
			UIFrameFadeRemoveFrame(EP_VoileMortelFrame)
			UIFrameFadeOut(EP_VoileMortelFrame, 0.5, EP_VoileMortelFrame:GetAlpha(), 0)

		elseif health > self.db.profile.health2 then
			if self.Flash ~= 1 then
				UIFrameFlashRemoveFrame(EP_VoileMortelFrame)
				UIFrameFadeRemoveFrame(EP_VoileMortelFrame)
				UIFrameFadeOut(EP_VoileMortelFrame, 0, EP_VoileMortelFrame:GetAlpha(), 0)
			end

			self.Flash = 1
			UIFrameFlash(EP_VoileMortelFrame, self.so_fit, self.so_fot, 3600, nil, self.so_fiht, self.so_foht)

		else
			if self.Flash ~= 2 then
				UIFrameFlashRemoveFrame(EP_VoileMortelFrame)
				UIFrameFadeRemoveFrame(EP_VoileMortelFrame)
				UIFrameFadeOut(EP_VoileMortelFrame, 0, EP_VoileMortelFrame:GetAlpha(), 0)
			end

			self.Flash = 2
			UIFrameFlash(EP_VoileMortelFrame, self.st_fit, self.st_fot, 3600, nil, self.st_fiht, self.st_foht)

		end
	end
end


--[[--------------------------------------------------------------------------------
  Options
-----------------------------------------------------------------------------------]]

function EP_VoileMortel:OptionShow(value)
	if type(value) == "nil" then return self.db.profile.show end
	self.db.profile.show = value
end

function EP_VoileMortel:OptionLayout(value)
	if type(value) == "nil" then return self.db.profile.layout end
	self.db.profile.layout = value
  self:SetMask()
end

function EP_VoileMortel:OptionHealth1(value)
	if type(value) == "nil" then return self.db.profile.health1 end
	self.db.profile.health1 = value
end

function EP_VoileMortel:OptionHealth2(value)
	if type(value) == "nil" then return self.db.profile.health2 end
	self.db.profile.health2 = value
end


--[[--------------------------------------------------------------------------------
  Menu
-----------------------------------------------------------------------------------]]

function EP_VoileMortel:InitMenu()
	self.dewdrop = AceLibrary("Dewdrop-2.0")

	self.ddoptions = {
		type= 'group',
		args = {
			show = {
				name = "show",
				type = "text",
				desc = "Show VoileMortel in combat or always",
        get	= function() return self:OptionShow() end,
        set	= function(v) return self:OptionShow(v) end,
				validate = {"always", "combat"}
			},
			layout = {
				name = "layout",
				type = "text",
				desc = "Select a layout",
        get	= function() return self:OptionLayout() end,
        set	= function(v) return self:OptionLayout(v) end,
				validate = {"square", "side", "full"}
			},
      health1 = {
        name = "health1",
        type = "range",
        desc = "First health trigger",
        get	= function() return self:OptionHealth1() end,
        set	= function(v) return self:OptionHealth1(v) end,
        min = 20,
        max = 90,
				step = 5
      },
      health2 = {
        name = "health2",
        type = "range",
        desc = "Second health trigger",
        get	= function() return self:OptionHealth2() end,
        set	= function(v) return self:OptionHealth2(v) end,
        min = 10,
        max = 80,
				step = 5
      },
		}
	}

	self.slashOptions = {
		type = "group",
		args = {
			config = {
				name = "config",
				desc = "openconfig",
				type = "execute",
				func = function()
					self.dewdrop:InjectAceOptionsTable(self, self.ddoptions)
					self.dewdrop:Register(UIParent, 'children', self.ddoptions, 'cursorX', true, 'cursorY', true)
					self.dewdrop:Open(UIParent)
				end,
			}
		}
	}

	self:RegisterChatCommand({ "/epvm" }, self.slashOptions )
end

