--[[--------------------------------------------------------------------------------
  Class Setup
-----------------------------------------------------------------------------------]]


EasyPriest = AceLibrary("AceAddon-2.0"):new(
	"AceConsole-2.0",
	"AceEvent-2.0",
	"AceDB-2.0",
	"FuBarPlugin-2.0"
)

local T = AceLibrary("Tablet-2.0")

EasyPriest.hasIcon = "Interface\\Icons\\Spell_Holy_ElunesGrace.jpg"
EasyPriest.defaultMinimapPosition = 220
EasyPriest.cannotDetachTooltip = true

function EasyPriest:OnInitialize()
	self:RegisterDB("EasyPriestDB")
	self:RegisterDefaults("profile", {})
	self:InitMenu()
end


--[[--------------------------------------------------------------------------------
  Addon Enabling/Disabling
-----------------------------------------------------------------------------------]]

function EasyPriest:OnEnable()
	self:RegisterEvent("EP_ANNOUNCECAST_UPDATE")
end

function EasyPriest:OnDisable()
	if EP_AnnounceCast then
		EP_AnnounceCast:ToggleActive(false)
	end

	if EP_BestTarget then
		EP_BestTarget:ToggleActive(false)
	end

	if EP_SimpleCast then
		EP_SimpleCast:ToggleActive(false)
	end
end


--[[--------------------------------------------------------------------------------
  Internal Functions
-----------------------------------------------------------------------------------]]

local function copyTable(from)
	local to = new()
	for k,v in pairs(from) do to[k] = v end
	return to
end


--[[---------------------------------------------------------------------------------
  Event handlers
------------------------------------------------------------------------------------]]

function EasyPriest:EP_ANNOUNCECAST_UPDATE()
	self.ddoptions.args.AnnounceCast.args = EP_AnnounceCast.ddoptions.args
	self.dewdrop:InjectAceOptionsTable(self, self.ddoptions)
	self.dewdrop:Register(UIParent, 'children', self.ddoptions,'cursorX', true, 'cursorY', true)
end

function EasyPriest:InitSubMenu()
	if next(self.ddoptions.args) ~= nil then return end

	local ac = not EP_AnnounceCast
	local ac_args = {}
	if not ac then
		for k,v in pairs(EP_AnnounceCast.ddoptions.args) do ac_args[k] = v end
		ac_args["enabled"] = {
			name = "Enabled",
			type = "toggle",
			desc = "Suspend/resume this addon",
			get  = function() return EP_AnnounceCast:IsActive() end,
			set  = function() return EP_AnnounceCast:ToggleActive() end,
			order = 1
		}
		ac_args["header"] = {
			type = "header",
			order = 2
		}
	end

	local sc = not EP_SimpleCast
	local sc_args = {}
	if not sc then
		for k,v in pairs(EP_SimpleCast.ddoptions.args) do sc_args[k] = v end
		sc_args["enabled"] = {
			name = "Enabled",
			type = "toggle",
			desc = "Suspend/resume this addon",
			get  = function() return EP_SimpleCast:IsActive() end,
			set  = function() return EP_SimpleCast:ToggleActive() end,
			order = 1
		}
		sc_args["header"] = {
			type = "header",
			order = 2
		}
	end

	local vm = not EP_VoileMortel
	local vm_args = {}
	if not vm then
		for k,v in pairs(EP_VoileMortel.ddoptions.args) do vm_args[k] = v end
		vm_args["enabled"] = {
			name = "Enabled",
			type = "toggle",
			desc = "Suspend/resume this addon",
			get  = function() return EP_VoileMortel:IsActive() end,
			set  = function() return EP_VoileMortel:ToggleActive() end,
			order = 1
		}
		vm_args["header"] = {
			type = "header",
			order = 2
		}
	end

	self.ddoptions.args = {
		AnnounceCast = {
			name	= "AnnounceCast",
			type	= "group",
			desc	= "AnnounceCast setup",
			hidden = ac,
			args = ac_args
		},
		SimpleCast = {
			name	= "SimpleCast",
			type	= "group",
			desc	= "SimpleCast setup",
			hidden = sc,
			args = sc_args
		},
		VoileMortel = {
			name	= "VoileMortel",
			type	= "group",
			desc	= "VoileMortel setup",
			hidden = vm,
			args = vm_args
		}
	}
end

--[[---------------------------------------------------------------------------------
  Menu
------------------------------------------------------------------------------------]]

function EasyPriest:OnTooltipUpdate()
	T:SetHint("EasyPriest options.")
	self:InitSubMenu()
end

function EasyPriest:InitMenu()
	self.dewdrop = AceLibrary("Dewdrop-2.0")

	self.ddoptions = {
		type = "group",
		args = {
		}
	}

	self.slashOptions = {
		type = "group",
		args = {
			config = {
				name = "config",
				desc = "Open config menu",
				type = "execute",
				func = function()
					self:InitSubMenu()
					self.dewdrop:InjectAceOptionsTable(self, self.ddoptions)
					self.dewdrop:Register(UIParent, 'children', self.ddoptions, 'cursorX', true, 'cursorY', true)
					self.dewdrop:Open(UIParent)
				end,
			},
		}
	}

	self:RegisterChatCommand({"/ep", "/easypriest"}, self.slashOptions)
	self.OnMenuRequest = self.ddoptions
end
